@@ -73,6 +73,11 @@ import io.undertow.util.HeaderMap;
 import io.undertow.util.HttpString;
 import io.undertow.util.StatusCodes;
 
+import static io.undertow.protocols.http2.Http2Channel.AUTHORITY;
+import static io.undertow.protocols.http2.Http2Channel.METHOD;
+import static io.undertow.protocols.http2.Http2Channel.PATH;
+import static io.undertow.protocols.http2.Http2Channel.SCHEME;
+
 /**
  * A server connection. There is one connection per request
  *
@@ -328,6 +333,11 @@ public class Http2ServerConnection extends ServerConnection {
         DateUtils.addDateHeaderIfRequired(exchange);
         headers.add(STATUS, exchange.getStatusCode());
         Connectors.flattenCookies(exchange);
+        if(!Connectors.isEntityBodyAllowed(exchange)) {
+            //we are not allowed to send an entity body for some requests
+            exchange.getResponseHeaders().remove(Headers.CONTENT_LENGTH);
+            exchange.getResponseHeaders().remove(Headers.TRANSFER_ENCODING);
+        }
         return originalSinkConduit;
     }
 
@@ -389,7 +399,10 @@ public class Http2ServerConnection extends ServerConnection {
 
     @Override
     public boolean isPushSupported() {
-        return channel.isPushEnabled() && !exchange.getRequestHeaders().contains(Headers.X_DISABLE_PUSH);
+        return channel.isPushEnabled()
+                && !exchange.getRequestHeaders().contains(Headers.X_DISABLE_PUSH)
+                // push is not supported for already pushed streams, just for peer-initiated (odd) ids
+                && responseChannel.getStreamId() % 2 != 0;
     }
 
     @Override
@@ -406,10 +419,10 @@ public class Http2ServerConnection extends ServerConnection {
     public boolean pushResource(String path, HttpString method, HeaderMap requestHeaders, final HttpHandler handler) {
         HeaderMap responseHeaders = new HeaderMap();
         try {
-            requestHeaders.put(Http2ReceiveListener.METHOD, method.toString());
-            requestHeaders.put(Http2ReceiveListener.PATH, path.toString());
-            requestHeaders.put(Http2ReceiveListener.AUTHORITY, exchange.getHostAndPort());
-            requestHeaders.put(Http2ReceiveListener.SCHEME, exchange.getRequestScheme());
+            requestHeaders.put(METHOD, method.toString());
+            requestHeaders.put(PATH, path.toString());
+            requestHeaders.put(AUTHORITY, exchange.getHostAndPort());
+            requestHeaders.put(SCHEME, exchange.getRequestScheme());
 
             Http2HeadersStreamSinkChannel sink = channel.sendPushPromise(responseChannel.getStreamId(), requestHeaders, responseHeaders);
             Http2ServerConnection newConnection = new Http2ServerConnection(channel, sink, getUndertowOptions(), getBufferSize(), rootHandler);
